@@ -36,6 +36,11 @@
 #ifdef HAVE_SYS_STATVFS_H
 # include <sys/statvfs.h>
 #endif
+#ifdef __APPLE__
+# include <CoreFoundation/CoreFoundation.h>
+# include <sys/param.h>
+#endif
+
 #include <termios.h>
 #include <unistd.h>
 
@@ -333,6 +338,28 @@ static NTSTATUS set_dosdev_symlink( void *args )
     return status;
 }
 
+#ifdef __APPLE__
+static LONGLONG get_free_bytes_for_important_data(int fd)
+{
+    CFURLRef url = NULL;
+    CFNumberRef num = NULL;
+    char *path = NULL;
+    LONGLONG space = -1;
+
+    if (!(path = malloc( MAXPATHLEN ))) goto done;
+    if (fcntl( fd, F_GETPATH, path ) == -1) goto done;
+    if (!(url = CFURLCreateFromFileSystemRepresentation( NULL, (UInt8 *)path, strlen( path ), false ))) goto done;
+    if (!CFURLCopyResourcePropertyForKey( url, kCFURLVolumeAvailableCapacityForImportantUsageKey, &num, NULL )) goto done;
+    CFNumberGetValue( num, kCFNumberLongLongType, &space );
+
+done:
+    free( path );
+    if (url) CFRelease( url );
+    if (num) CFRelease( num );
+    return space;
+}
+#endif
+
 static NTSTATUS get_volume_size_info( void *args )
 {
     const struct get_volume_size_info_params *params = args;
@@ -350,6 +377,10 @@ static NTSTATUS get_volume_size_info( void *args )
     struct statfs stfs;
 #endif
 
+#ifdef __APPLE__
+    LONGLONG important_free_bytes;
+#endif
+
     if (!unix_mount) return STATUS_NO_SUCH_DEVICE;
 
     if (unix_mount[0] != '/')
@@ -387,6 +418,12 @@ static NTSTATUS get_volume_size_info( void *args )
     }
     bsize = stfs.f_bsize;
 #endif
+
+#ifdef __APPLE__
+    important_free_bytes = get_free_bytes_for_important_data( fd );
+    if (important_free_bytes != -1) stfs.f_bavail = stfs.f_bfree = important_free_bytes / bsize;
+#endif
+
     if (bsize == 2048)  /* assume CD-ROM */
     {
         info->bytes_per_sector = 2048;
