@@ -405,6 +405,7 @@ static const KBDTABLES kbdus_tables =
 };
 
 static LONG clipping_cursor; /* clipping thread counter */
+static LONG enable_mouse_in_pointer = -1;
 
 BOOL grab_pointer = TRUE;
 BOOL grab_fullscreen = FALSE;
@@ -800,7 +801,7 @@ SHORT WINAPI NtUserGetAsyncKeyState( INT key )
 
     if (key < 0 || key >= 256) return 0;
 
-    check_for_events( QS_INPUT );
+    check_for_events( QS_ALLINPUT );
 
     while ((status = get_shared_desktop( &lock, &desktop_shm )) == STATUS_PENDING)
         state = desktop_shm->keystate[key];
@@ -1964,6 +1965,9 @@ BOOL set_active_window( HWND hwnd, HWND *prev, BOOL mouse, BOOL focus, DWORD new
     DWORD winflags, old_thread, new_thread;
     CBTACTIVATESTRUCT cbt;
 
+    TRACE( "hwnd %p, previous %p, mouse %u, focus %u, new_active_thread_id %04x\n",
+           hwnd, previous, mouse, focus, new_active_thread_id );
+
     if (previous == hwnd)
     {
         if (prev) *prev = hwnd;
@@ -2206,6 +2210,8 @@ BOOL set_foreground_window( HWND hwnd, BOOL mouse, BOOL internal )
     }
     SERVER_END_REQ;
 
+    TRACE( "hwnd %p, mouse %u, internal %u -> ret %u, previous %p\n", hwnd, mouse, internal, ret, previous );
+
     if (ret && previous != hwnd)
     {
         if (send_msg_old)  /* old window belongs to other thread */
@@ -2588,15 +2594,22 @@ void toggle_caret( HWND hwnd )
     if (ret && !hidden) display_caret( hwnd, &r );
 }
 
-
 /**********************************************************************
  *       NtUserEnableMouseInPointer    (win32u.@)
  */
 BOOL WINAPI NtUserEnableMouseInPointer( BOOL enable )
 {
-    FIXME( "enable %u stub!\n", enable );
-    RtlSetLastWin32Error( ERROR_CALL_NOT_IMPLEMENTED );
-    return FALSE;
+    LONG prev;
+
+    TRACE( "enable %u\n", enable );
+
+    if ((prev = InterlockedCompareExchange( &enable_mouse_in_pointer, !!enable, -1 )) != -1 && prev != enable)
+    {
+        RtlSetLastWin32Error( ERROR_ACCESS_DENIED );
+        return FALSE;
+    }
+
+    return TRUE;
 }
 
 /**********************************************************************
@@ -2614,9 +2627,14 @@ BOOL WINAPI NtUserEnableMouseInPointerForThread( void )
  */
 BOOL WINAPI NtUserIsMouseInPointerEnabled(void)
 {
-    FIXME( "stub!\n" );
-    RtlSetLastWin32Error( ERROR_CALL_NOT_IMPLEMENTED );
-    return FALSE;
+    BOOL ret = ReadNoFence( &enable_mouse_in_pointer ) == 1;
+    TRACE( "-> %d.\n", ret );
+    return ret;
+}
+
+BOOL is_mouse_in_pointer_enabled( HWND hwnd )
+{
+    return ReadNoFence( &enable_mouse_in_pointer ) == 1;
 }
 
 static BOOL is_captured_by_system(void)
@@ -2656,7 +2674,8 @@ BOOL clip_fullscreen_window( HWND hwnd, BOOL reset )
     if ((style & WS_MAXIMIZE) && (style & WS_CAPTION) == WS_CAPTION) return FALSE;
 
     ctx = set_thread_dpi_awareness_context( NTUSER_DPI_PER_MONITOR_AWARE );
-    ret = get_window_rect( hwnd, &window_rect, get_thread_dpi() );
+    if (!(ret = get_present_rect( hwnd, &window_rect, get_thread_dpi() )))
+        ret = get_window_rect( hwnd, &window_rect, get_thread_dpi() );
     monitor_info = monitor_info_from_window( hwnd, MONITOR_DEFAULTTONEAREST );
     virtual_rect = get_virtual_screen_rect( get_thread_dpi(), MDT_DEFAULT );
     monitor_rect = map_rect_virt_to_raw( monitor_info.rcMonitor, get_thread_dpi() );
@@ -2762,6 +2781,7 @@ BOOL WINAPI NtUserGetClipCursor( RECT *rect )
  */
 BOOL WINAPI NtUserClipCursor( const RECT *rect )
 {
+    UINT dpi = get_thread_dpi();
     RECT new_rect;
     BOOL ret;
 
@@ -2769,8 +2789,14 @@ BOOL WINAPI NtUserClipCursor( const RECT *rect )
 
     if (rect)
     {
+        HWND foreground = NtUserGetForegroundWindow();
+        if (IsRectEmpty( rect ) && get_present_rect( foreground, &new_rect, dpi ))
+        {
+            WARN( "Fullscreen clipping fixup to %s\n", wine_dbgstr_rect(&new_rect) );
+            rect = &new_rect;
+        }
         if (rect->left > rect->right || rect->top > rect->bottom) return FALSE;
-        new_rect = map_rect_virt_to_raw( *rect, get_thread_dpi() );
+        new_rect = map_rect_virt_to_raw( *rect, dpi );
         rect = &new_rect;
     }
 
