@@ -485,7 +485,7 @@ static void disable_system_cursors(void)
 /***********************************************************************
  *           x11drv_tablet_load_info
  */
-NTSTATUS x11drv_tablet_load_info( void *hwnd )
+static BOOL load_tablet_info( HWND hwnd )
 {
     static const WCHAR SZ_CONTEXT_NAME[] = {'W','i','n','e',' ','T','a','b','l','e','t',' ','C','o','n','t','e','x','t',0};
     static const WCHAR SZ_DEVICE_NAME[] = {'W','i','n','e',' ','T','a','b','l','e','t',' ','D','e','v','i','c','e',0};
@@ -1019,9 +1019,9 @@ static BOOL proximity_event( HWND hwnd, XEvent *event )
 }
 
 /***********************************************************************
- *           x11drv_tablet_attach_queue
+ *           tablet_attach_queue
  */
-NTSTATUS x11drv_tablet_attach_queue( void *owner )
+static BOOL tablet_attach_queue( HWND owner )
 {
     struct x11drv_thread_data *data = x11drv_init_thread_data();
     int             num_devices;
@@ -1033,7 +1033,7 @@ NTSTATUS x11drv_tablet_attach_queue( void *owner )
     XEventClass     event_list[7];
     Window          win = X11DRV_get_whole_window( owner );
 
-    if (!win || !xinput_handle) return 0;
+    if (!win || !xinput_handle) return FALSE;
 
     TRACE("Creating context for window %p (%lx)  %i cursors\n", owner, win, gNumCursors);
 
@@ -1107,19 +1107,9 @@ NTSTATUS x11drv_tablet_attach_queue( void *owner )
     X11DRV_check_error();
 
     if (NULL != devices) pXFreeDeviceList(devices);
-    return 0;
+    return TRUE;
 }
 
-/***********************************************************************
- *           x11drv_tablet_get_packet
- */
-NTSTATUS x11drv_tablet_get_packet( void *packet )
-{
-    *(WTPACKET *)packet = gMsgPacket;
-    return 1;
-}
-
-
 static inline int CopyTabletData(LPVOID target, LPCVOID src, INT size)
 {
     /*
@@ -1132,15 +1122,10 @@ static inline int CopyTabletData(LPVOID target, LPCVOID src, INT size)
 }
 
 /***********************************************************************
- *           x11drv_tablet_info
+ *           get_tablet_info
  */
-NTSTATUS x11drv_tablet_info( void *arg )
+static UINT get_tablet_info( UINT wCategory, UINT nIndex, void *lpOutput )
 {
-    struct tablet_info_params *params = arg;
-    UINT wCategory = params->category;
-    UINT nIndex = params->index;
-    void *lpOutput = params->output;
-
     /*
      * It is valid to call WTInfoA with lpOutput == NULL, as per standard.
      * lpOutput == NULL signifies the user only wishes
@@ -1633,20 +1618,32 @@ NTSTATUS x11drv_tablet_info( void *arg )
     return rc;
 }
 
-#else /* SONAME_LIBXI */
-
 /***********************************************************************
- *           x11drv_tablet_attach_queue
+ *           X11DRV_WintabProc
  */
-NTSTATUS x11drv_tablet_attach_queue( void *owner )
+LRESULT X11DRV_WintabProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam, void *buffer )
 {
+    switch (msg)
+    {
+    case NtUserWintabAttach:
+        return tablet_attach_queue( hwnd );
+    case NtUserWintabInfo:
+        return get_tablet_info( wparam, lparam, buffer );
+    case NtUserWintabInit:
+        return load_tablet_info( hwnd );
+    case NtUserWintabPacket:
+        *(WTPACKET *)buffer = gMsgPacket;
+        return 1;
+    }
     return 0;
 }
 
+#else /* SONAME_LIBXI */
+
 /***********************************************************************
- *           x11drv_tablet_get_packet
+ *           X11DRV_WintabProc
  */
-NTSTATUS x11drv_tablet_get_packet( void *arg )
+LRESULT X11DRV_WintabProc( HWND hwmd, UINT msg, WPARAM wparam, LPARAM lparam, void *buffer )
 {
     return 0;
 }
